;; Generated package description from realgud.el
(define-package "realgud" "1.4.3" "A modular front-end for interacting with external debuggers" '((load-relative "1.2") (loc-changes "1.2") (test-simple "1.2.0")) :url "http://github.com/realgud/realgud/" :keywords nil)
